/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package es.rickyepoderi.snmpjmxagent;

import java.io.IOException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import org.snmp4j.agent.MOGroup;
import org.snmp4j.agent.ManagedObject;
import org.snmp4j.agent.mo.MOGroupImpl;
import org.snmp4j.agent.mo.jmx.JMXDefaultMOFactory;
import org.snmp4j.agent.mo.jmx.MBeanAttributeMOScalarSupport;

/**
 * <p>Singleton object that runs the SNMP agent server and interacts with in
 * order to register and unregister MBeans. The agent maps the attributes of
 * a MBean into a SNMP oid and type.</p>
 * 
 * @author ricky
 */
public enum SnmpJmxModule {
    
    /**
     * The singleton instance.
     */
    INSTANCE;

    /**
     * The SNMP server agent.
     */
    private SnmpAgent agent;
    
    /**
     * Helper method to get the instance. Obviously SnmpJmxModule.INSTANCE is
     * exactly the same.
     * 
     * @return The singleton object
     */
    static public SnmpJmxModule getInstance() {
        return INSTANCE;
    }
    
    
    /**
     * Private constructor that initializes the SNMP agent in localhost.
     */
    private SnmpJmxModule() {
        try {
            // create and start the agent
            agent = new SnmpAgent("127.0.0.1/1161");
            agent.start();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Method that registers a MBean object into the SNMP server.
     * 
     * @param server The JMS server.
     * @param name The name of the object already registered in the previous server.
     * @param descs The mapping description between JMX and SNMP.
     * @return The MOGroup registered in the SNMP agent for later un-registration.
     */
    public MOGroup registerBean(MBeanServer server, ObjectName name, ScalarAttributeDescription[] descs) {
        MOGroupImpl mog = new MOGroupImpl();
        // register the scalar into the SNMP agent
        MBeanAttributeMOScalarSupport scalarSupport = new MBeanAttributeMOScalarSupport(server);
        // convert the ScalarAttributeDescription to SCALAR INFO
        Object[][] scalar = new Object[descs.length][3];
        for (int i = 0; i < descs.length; i++) {
            scalar[i][0] = descs[i].getOid();
            scalar[i][1] = descs[i].getAttributeName();
            scalar[i][2] = descs[i].getAttributeType();
        }
        // register the scalar
        scalarSupport.addAll(name, scalar);
        // create the jmxFactory and assing all the attributes
        JMXDefaultMOFactory jmxFactory = new JMXDefaultMOFactory(server, scalarSupport);
        for (ScalarAttributeDescription d: descs) {
            ManagedObject mo = jmxFactory.createScalar(d.getOid(), d.getAccess(), d.getVariable());
            mog.addInstance(mo);
        }
        // register the group
        agent.registerManagedObject(mog);
        return mog;
    }
    
    /**
     * Method that unregisters a previous registered MOGroup.
     * 
     * @param mog The MOGroup to unregister,
     */
    public void unregisterBean(MOGroup mog) {
        agent.unregisterManagedObject(mog);
    }
    
}
